import os
import subprocess
from setuptools import setup, find_packages
import pathlib

# Caminho para o README.md
current_dir = pathlib.Path(__file__).parent.resolve()
long_description = (current_dir / "README.md").read_text(encoding="utf-8")

import platform
import sys


def install_dependencies():
    try:
        import jpype
        print("JPype1 já está instalado.")
    except ImportError:
        import platform
        import sys
        import subprocess
        import os

        system = platform.system().lower()
        python_version = f"cp{sys.version_info.major}{sys.version_info.minor}"
        dependencies_dir = os.path.abspath(
            os.path.join(os.path.dirname(__file__), "wbjdbc", "resources", "dependencies")
        )

        if system == "windows":
            wheel_file = f"jpype1-1.5.1-{python_version}-{python_version}-win_amd64.whl"
        elif system == "linux":
            wheel_file = f"jpype1-1.5.1-{python_version}-{python_version}-manylinux_2_17_x86_64.manylinux2014_x86_64.whl"
        elif system == "darwin":
            wheel_file = f"jpype1-1.5.1-{python_version}-{python_version}-macosx_10_9_universal2.whl"
        else:
            raise RuntimeError(f"Sistema operacional não suportado: {system}")

        wheel_path = os.path.join(dependencies_dir, wheel_file)

        # Primeira tentativa: PyPI
        try:
            print("Tentando instalar JPype1 diretamente do PyPI...")
            subprocess.check_call(["pip", "install", "JPype1"])
        except subprocess.CalledProcessError as e:
            print(f"Erro ao instalar JPype1 do PyPI: {e}")

            # Tentativa local se PyPI falhar
            if os.path.exists(wheel_path):
                print(f"Tentando instalar JPype1 localmente de {wheel_path} para {system}.")
                try:
                    subprocess.check_call(["pip", "install", wheel_path])
                except subprocess.CalledProcessError as e:
                    print(f"Erro ao instalar JPype1 localmente de {wheel_path}: {e}")
                    raise RuntimeError("Falha na instalação do JPype1.")
            else:
                print(f"Arquivo {wheel_file} não encontrado no diretório {dependencies_dir}.")
                raise RuntimeError("Falha na instalação do JPype1.")
    finally:
        # Garantir que JPype1 foi instalado
        try:
            import jpype
            print("JPype1 instalado com sucesso!")
        except ImportError:
            print("Erro: JPype1 não pôde ser instalado.")
            raise RuntimeError("Falha final na instalação do JPype1.")



# Chama a função para instalar JPype1
install_dependencies()

setup(
    name="wbjdbc",
    version="1.0.6",
    packages=find_packages(),
    include_package_data=True,
    package_data={
        "wbjdbc": [
            "resources/server/**",
            "resources/maven/com.ibm.informix/*",
            "resources/maven/org.mongodb/*",
            "resources/dependencies/*",
        ]
    },
    install_requires=[],  # Não lista JPype1 aqui, pois ele é instalado manualmente
    description="Library to simplify JDBC and JVM configuration for Informix and MongoDB",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Wanderson Batista",
    author_email="wanderfreitasb@gmail.com",
    url="https://github.com/wanderbatistaf/wbjdbc",
)



import os
import sys
import subprocess
import jpype
from pkg_resources import resource_filename


class JVMError(Exception):
    """Classe personalizada para erros de JVM."""
    pass


def ensure_jpype_installed():
    """Garante que JPype1 seja instalado a partir do wheel interno."""
    try:
        import jpype
    except ImportError:
        # Localiza o diretório de wheels dentro do pacote
        wheels_dir = resource_filename("wbjdbc", "wheels")

        # Verifica a existência do diretório de wheels
        if not os.path.isdir(wheels_dir):
            raise JVMError(f"Diretório de wheels não encontrado: {wheels_dir}")

        # Localiza o arquivo JPype1 no diretório de wheels
        wheel_file = next((f for f in os.listdir(wheels_dir) if "JPype1" in f and f.endswith(".whl")), None)
        if not wheel_file:
            raise JVMError("Wheel do JPype1 não encontrado no diretório de wheels.")

        # Instala o JPype1 a partir do wheel interno
        try:
            subprocess.check_call([sys.executable, "-m", "pip", "install", os.path.join(wheels_dir, wheel_file)])
            print("JPype1 instalado com sucesso.")
        except subprocess.CalledProcessError as e:
            raise JVMError(f"Falha ao instalar JPype1: {e}")


# Garante que JPype1 está instalado antes de prosseguir
ensure_jpype_installed()


def start_jvm():
    """Inicia a JVM com os recursos internos."""
    try:
        # Localiza os arquivos necessários
        jvm_path = resource_filename("wbjdbc", "resources/server/jdk-17.0.2/bin/server/jvm.dll")
        jdbc_driver_path = resource_filename("wbjdbc", "resources/maven/com.ibm.informix/jdbc-4.50.10.1.jar")
        bson_jar_path = resource_filename("wbjdbc", "resources/maven/org.mongodb/bson-3.8.0.jar")

        # Verifica a existência dos arquivos
        for file_path, desc in [
            (jvm_path, "JVM"),
            (jdbc_driver_path, "JDBC Driver"),
            (bson_jar_path, "BSON Jar"),
        ]:
            if not os.path.isfile(file_path):
                raise JVMError(f"Arquivo {desc} não encontrado: {file_path}")

        # Configuração do classpath
        classpath = f"{jdbc_driver_path};{bson_jar_path}"

        # Inicializa a JVM
        if not jpype.isJVMStarted():
            jpype.startJVM(jvm_path, f"-Djava.class.path={classpath}")
            print("JVM inicializada com sucesso!")
        else:
            print("JVM já está inicializada.")
    except JVMError as e:
        print(f"Erro na inicialização da JVM: {e}")
        raise
    except jpype.JVMNotSupportedException as e:
        print(f"A JVM não é suportada: {e}")
        raise
    except Exception as e:
        print(f"Erro inesperado ao inicializar a JVM: {e}")
        raise
